@@ -1,10 +1,12 @@
 {-# LANGUAGE Arrows #-}
 module Skylighting.Parser ( parseSyntaxDefinition
+                          , parseSyntaxDefinitionFromString
                           , addSyntaxDefinition
                           , missingIncludes
                           ) where
 
-import Data.ByteString.UTF8 (fromString)
+import Data.ByteString.UTF8 (fromString, toString)
+import qualified Data.ByteString as BS
 import Data.Char (isAlphaNum, toUpper)
 import qualified Data.Map as Map
 import Data.Maybe (fromMaybe)
@@ -64,22 +66,42 @@ vBool defaultVal value = case value of
                            _ -> defaultVal
 
 -- | Parses a file containing a Kate XML syntax definition
--- into a 'Syntax' description.
-parseSyntaxDefinition :: String -> IO (Either String Syntax)
-parseSyntaxDefinition xml = do
-  res <- runX (application xml)
+-- into a 'Syntax' description.  Note that if the DOCTYPE contains
+-- a reference to the now-obsolete language.dtd, we remove it.
+parseSyntaxDefinition :: FilePath -> IO (Either String Syntax)
+parseSyntaxDefinition fp = do
+  xml <- toString <$> BS.readFile fp
+  parseSyntaxDefinitionFromString fp xml
+
+-- | Parses a string containing a Kate XML syntax definition
+-- into a 'Syntax' description.  Note that if the DOCTYPE contains
+-- a reference to the now-obsolete language.dtd, we remove it.
+parseSyntaxDefinitionFromString :: FilePath -- ^ used for short name
+                                -> String
+                                -> IO (Either String Syntax)
+parseSyntaxDefinitionFromString fp xml = do
+  res <- runX ( readString [withValidate no]
+                  (removeLanguageDTD . removeBOM $ xml)
+                >>>
+                application fp )
   case res of
        [s] -> return $ Right s
-       _   -> return $ Left $ "Could not parse syntax definition " ++ xml
+       _   -> return $ Left $ "Could not parse syntax definition " ++ fp
 
-application :: String -> IOSArrow b Syntax
-application fp@(_:':':'\\':_) =
-  -- Windows C:/, so HXT doesn't interpret as a URI scheme.
-  application ("file:///" ++ map (\c -> if c == '\\' then '/' else c) fp)
+removeBOM :: String -> String
+removeBOM ('\xFEFF':xs) = xs
+removeBOM xs            = xs
+
+removeLanguageDTD :: String -> String
+removeLanguageDTD ('S':'Y':'S':'T':'E':'M':' ':xs) =
+  removeLanguageDTD $ dropWhile (\c -> c /= '[' && c /= '>') xs
+removeLanguageDTD xs@('<':'l':'a':'n':'g':_) = xs
+removeLanguageDTD (x:xs) = x : removeLanguageDTD xs
+removeLanguageDTD [] = []
+
+application :: FilePath -> IOSArrow XmlTree Syntax
 application fp
-    = readDocument [withValidate no, withInputEncoding utf8] fp
-      >>>
-      multi (hasName "language")
+    = multi (hasName "language")
       >>>
       extractSyntaxDefinition (takeFileName fp)
 
